
# Copyright (C) 2018-2019 Vanessa Sochat.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



__version__ = "0.0.11"
AUTHOR = 'Vanessa Sochat'
AUTHOR_EMAIL = 'vsochat@stanford.edu'
NAME = 'sif'
PACKAGE_URL = "http://www.github.com/singularityhub/sif"
KEYWORDS = 'container, singularity, sif'
DESCRIPTION = "python handle for reading sif images"
LICENSE = "LICENSE"

INSTALL_REQUIRES = (
)
